const wordCount = {};
const src = read("texts/flatland.txt");
const txt = src
    .split(/(\s+|[.,!?:])/)
    .filter(w=>w.replace(/[ \t]+/g, "").length);

txt.forEach((word, i)=>{
    word = word.trim();
    word = word.replace(/\\/g, "\\\\");
    if(!word)
        word = '\\n';
    txt[i] = word;
    wordCount[word] = (wordCount[word]|0) + 1;
});

const wordList = Object.keys(wordCount)
    .map(word=>({
        word, 
        count:wordCount[word]
    }))
    .sort((a, b)=>{
        return b.count - a.count;
    })
    .map(entry=>entry.word);
    
const wordIndex = {};
wordList.forEach((w, i) => wordIndex[w] = pack(i)); // .map(x=>`0x`+x.toString(16)).join(",") + `/*${i}*/`);

const out = [];
txt.forEach(word=>out.push(...wordIndex[word]));

write("texts/words.h", `
static const char words[] = 
${wordList.map((w, i)=>`"${w.replace(/"/g, "\\\"")}\\0" // ${i}\n`).join("")};
`);
write("texts/indices.h", `
static const char indices[] = {
${txt.map(w=>wordIndex[w] + ", // " + w).join("\n")}
};
`)

let total = wordList.join("\n").length + out.length;

log("Word count: " + wordList[1170]);
log("Dictionary size: " + wordList.join("\n").length);
log("Text size: " + out.length);
log("Total size: " + total);
log("Source size: " + src.length);

function pack(w){
    if(!w) 
        return [0];
    
    let b = [];
    while(w){
        let c = w & 0x7F;
        w >>= 7;
        if(w) c |= 0x80;
        b.push(c);
    }
    
    return b;
}